@@ -1,9 +1,13 @@
+#include "iperf_config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <sysexits.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 
 #include <iperf_api.h>
 
@@ -44,12 +48,18 @@ main( int argc, char** argv )
     iperf_set_test_duration( test, 5 );
     iperf_set_test_reporter_interval( test, 1 );
     iperf_set_test_stats_interval( test, 1 );
+    /* iperf_set_test_json_output( test, 1 ); */
 
     if ( iperf_run_client( test ) < 0 ) {
 	fprintf( stderr, "%s: error - %s\n", argv0, iperf_strerror( i_errno ) );
 	exit( EXIT_FAILURE );
     }
 
+    if (iperf_get_test_json_output_string(test)) {
+	fprintf(iperf_get_test_outfile(test), "%zd bytes of JSON emitted\n",
+		strlen(iperf_get_test_json_output_string(test)));
+    }
+
     iperf_free_test( test );
     exit( EXIT_SUCCESS );
 }
