@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014, The Regents of the University of
+ * iperf, Copyright (c) 2014, 2015, 2016, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -27,11 +27,22 @@
 #ifndef __IPERF_H
 #define __IPERF_H
 
+#include "iperf_config.h"
+
 #include <sys/time.h>
 #include <sys/types.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+#include <sys/select.h>
 #include <sys/socket.h>
 #include <netinet/tcp.h>
+
+#if defined(HAVE_CPUSET_SETAFFINITY)
+#include <sys/param.h>
+#include <sys/cpuset.h>
+#endif /* HAVE_CPUSET_SETAFFINITY */
+
 #include "timer.h"
 #include "queue.h"
 #include "cjson.h"
@@ -55,8 +66,9 @@ struct iperf_interval_results
     int       cnt_error;
 
     int omitted;
-#if defined(linux) || defined(__FreeBSD__)
-    struct tcp_info tcpInfo;	/* getsockopt(TCP_INFO) for Linux and FreeBSD */
+#if (defined(linux) || defined(__FreeBSD__) || defined(__NetBSD__)) && \
+	defined(TCP_INFO)
+    struct tcp_info tcpInfo; /* getsockopt(TCP_INFO) for Linux, {Free,Net}BSD */
 #else
     /* Just placeholders, never accessed. */
     char *tcpInfo;
@@ -66,6 +78,7 @@ struct iperf_interval_results
     int snd_cwnd;
     TAILQ_ENTRY(iperf_interval_results) irlistentries;
     void     *custom_data;
+    int rtt;
 };
 
 struct iperf_stream_result
@@ -74,12 +87,19 @@ struct iperf_stream_result
     iperf_size_t bytes_sent;
     iperf_size_t bytes_received_this_interval;
     iperf_size_t bytes_sent_this_interval;
+    iperf_size_t bytes_sent_omit;
     int stream_prev_total_retrans;
     int stream_retrans;
     int stream_prev_total_sacks;
     int stream_sacks;
+    int stream_max_rtt;
+    int stream_min_rtt;
+    int stream_sum_rtt;
+    int stream_count_rtt;
+    int stream_max_snd_cwnd;
     struct timeval start_time;
     struct timeval end_time;
+    struct timeval start_time_fixed;
     TAILQ_HEAD(irlisthead, iperf_interval_results) interval_results;
     void     *data;
 };
@@ -97,8 +117,9 @@ struct iperf_settings
     int       tos;                  /* type of service bit */
     int       flowlabel;            /* IPv6 flow label */
     iperf_size_t bytes;             /* number of bytes to send */
-    int       blocks;               /* number of blocks (packets) to send */
+    iperf_size_t blocks;            /* number of blocks (packets) to send */
     char      unit_format;          /* -f */
+    int       num_ostreams;         /* SCTP initmsg settings */
 };
 
 struct iperf_test;
@@ -133,7 +154,9 @@ struct iperf_stream
     double    jitter;
     double    prev_transit;
     int       outoforder_packets;
+    int       omitted_outoforder_packets;
     int       cnt_error;
+    int       omitted_cnt_error;
     uint64_t  target;
 
     struct sockaddr_storage local_addr;
@@ -169,6 +192,12 @@ struct iperf_textline {
     TAILQ_ENTRY(iperf_textline) textlineentries;
 };
 
+struct xbind_entry {
+    char *name;
+    struct addrinfo *ai;
+    TAILQ_ENTRY(xbind_entry) link;
+};
+
 struct iperf_test
 {
     char      role;                             /* 'c' lient or 's' erver */
@@ -177,14 +206,24 @@ struct iperf_test
     struct protocol *protocol;
     signed char state;
     char     *server_hostname;                  /* -c option */
-    char     *bind_address;                     /* -B option */
+    char     *tmp_template;
+    char     *bind_address;                     /* first -B option */
+    TAILQ_HEAD(xbind_addrhead, xbind_entry) xbind_addrs; /* all -X opts */
+    int       bind_port;                        /* --cport option */
     int       server_port;
     int       omit;                             /* duration of omit period (-O flag) */
     int       duration;                         /* total duration of test (-t flag) */
     char     *diskfile_name;			/* -F option */
     int       affinity, server_affinity;	/* -A option */
+#if defined(HAVE_CPUSET_SETAFFINITY)
+    cpuset_t cpumask;
+#endif /* HAVE_CPUSET_SETAFFINITY */
     char     *title;				/* -T option */
     char     *congestion;			/* -C option */
+    char     *pidfile;				/* -P option */
+
+    char     *logfile;				/* --logfile option */
+    FILE     *outfile;
 
     int       ctrl_sck;
     int       listener;
@@ -200,9 +239,11 @@ struct iperf_test
     int	      zerocopy;                         /* -Z option - use sendfile */
     int       debug;				/* -d option - enable debug */
     int	      get_server_output;		/* --get-server-output */
-
+    int	      udp_counters_64bit;		/* --use-64-bit-udp-counters */
+    int       no_fq_socket_pacing;	  /* --no-fq-socket-pacing */
     int	      multisend;
 
+    char     *json_output_string; /* rendered JSON output if json_output is set */
     /* Select related parameters */
     int       max_fd;
     fd_set    read_set;                         /* set of read sockets */
@@ -226,7 +267,7 @@ struct iperf_test
     int       num_streams;                      /* total streams in the test (-P) */
 
     iperf_size_t bytes_sent;
-    int       blocks_sent;
+    iperf_size_t blocks_sent;
     char      cookie[COOKIE_SIZE];
 //    struct iperf_stream *streams;               /* pointer to list of struct stream */
     SLIST_HEAD(slisthead, iperf_stream) streams;
