@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014, The Regents of the University of
+ * iperf, Copyright (c) 2014, 2016, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -33,7 +33,9 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <netinet/in.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 #include <sys/time.h>
 #include <sys/select.h>
 
@@ -43,7 +45,7 @@
 #include "iperf_udp.h"
 #include "timer.h"
 #include "net.h"
-
+#include "portable_endian.h"
 
 /* iperf_udp_recv
  *
@@ -52,9 +54,10 @@
 int
 iperf_udp_recv(struct iperf_stream *sp)
 {
+    uint32_t  sec, usec;
+    uint64_t  pcount;
     int       r;
     int       size = sp->settings->blksize;
-    uint32_t  sec, usec, pcount;
     double    transit = 0, d = 0;
     struct timeval sent_time, arrival_time;
 
@@ -71,14 +74,27 @@ iperf_udp_recv(struct iperf_stream *sp)
     sp->result->bytes_received += r;
     sp->result->bytes_received_this_interval += r;
 
-    memcpy(&sec, sp->buffer, sizeof(sec));
-    memcpy(&usec, sp->buffer+4, sizeof(usec));
-    memcpy(&pcount, sp->buffer+8, sizeof(pcount));
-    sec = ntohl(sec);
-    usec = ntohl(usec);
-    pcount = ntohl(pcount);
-    sent_time.tv_sec = sec;
-    sent_time.tv_usec = usec;
+    if (sp->test->udp_counters_64bit) {
+	memcpy(&sec, sp->buffer, sizeof(sec));
+	memcpy(&usec, sp->buffer+4, sizeof(usec));
+	memcpy(&pcount, sp->buffer+8, sizeof(pcount));
+	sec = ntohl(sec);
+	usec = ntohl(usec);
+	pcount = be64toh(pcount);
+	sent_time.tv_sec = sec;
+	sent_time.tv_usec = usec;
+    }
+    else {
+	uint32_t pc;
+	memcpy(&sec, sp->buffer, sizeof(sec));
+	memcpy(&usec, sp->buffer+4, sizeof(usec));
+	memcpy(&pc, sp->buffer+8, sizeof(pc));
+	sec = ntohl(sec);
+	usec = ntohl(usec);
+	pcount = ntohl(pc);
+	sent_time.tv_sec = sec;
+	sent_time.tv_usec = usec;
+    }
 
     /* Out of order packets */
     if (pcount >= sp->packet_count + 1) {
@@ -88,7 +104,7 @@ iperf_udp_recv(struct iperf_stream *sp)
         sp->packet_count = pcount;
     } else {
         sp->outoforder_packets++;
-	iperf_err(sp->test, "OUT OF ORDER - incoming packet = %d and received packet = %d AND SP = %d", pcount, sp->packet_count, sp->socket);
+	iperf_err(sp->test, "OUT OF ORDER - incoming packet = %zu and received packet = %d AND SP = %d", pcount, sp->packet_count, sp->socket);
     }
 
     /* jitter measurement */
@@ -103,6 +119,10 @@ iperf_udp_recv(struct iperf_stream *sp)
     //      J = |(R1 - S1) - (R0 - S0)| [/ number of packets, for average]
     sp->jitter += (d - sp->jitter) / 16.0;
 
+    if (sp->test->debug) {
+	fprintf(stderr, "packet_count %d\n", sp->packet_count);
+    }
+
     return r;
 }
 
@@ -115,20 +135,40 @@ int
 iperf_udp_send(struct iperf_stream *sp)
 {
     int r;
-    uint32_t  sec, usec, pcount;
     int       size = sp->settings->blksize;
     struct timeval before;
 
     gettimeofday(&before, 0);
 
     ++sp->packet_count;
-    sec = htonl(before.tv_sec);
-    usec = htonl(before.tv_usec);
-    pcount = htonl(sp->packet_count);
 
-    memcpy(sp->buffer, &sec, sizeof(sec));
-    memcpy(sp->buffer+4, &usec, sizeof(usec));
-    memcpy(sp->buffer+8, &pcount, sizeof(pcount));
+    if (sp->test->udp_counters_64bit) {
+
+	uint32_t  sec, usec;
+	uint64_t  pcount;
+
+	sec = htonl(before.tv_sec);
+	usec = htonl(before.tv_usec);
+	pcount = htobe64(sp->packet_count);
+	
+	memcpy(sp->buffer, &sec, sizeof(sec));
+	memcpy(sp->buffer+4, &usec, sizeof(usec));
+	memcpy(sp->buffer+8, &pcount, sizeof(pcount));
+	
+    }
+    else {
+
+	uint32_t  sec, usec, pcount;
+
+	sec = htonl(before.tv_sec);
+	usec = htonl(before.tv_usec);
+	pcount = htonl(sp->packet_count);
+	
+	memcpy(sp->buffer, &sec, sizeof(sec));
+	memcpy(sp->buffer+4, &usec, sizeof(usec));
+	memcpy(sp->buffer+8, &pcount, sizeof(pcount));
+	
+    }
 
     r = Nwrite(sp->socket, sp->buffer, size, Pudp);
 
@@ -204,6 +244,23 @@ iperf_udp_accept(struct iperf_test *test)
         }
     }
 
+#if defined(HAVE_SO_MAX_PACING_RATE)
+    /* If socket pacing is available and not disabled, try it. */
+    if (! test->no_fq_socket_pacing) {
+	/* Convert bits per second to bytes per second */
+	unsigned int rate = test->settings->rate / 8;
+	if (rate > 0) {
+	    if (test->debug) {
+		printf("Setting fair-queue socket pacing to %u\n", rate);
+	    }
+	    if (setsockopt(s, SOL_SOCKET, SO_MAX_PACING_RATE, &rate, sizeof(rate)) < 0) {
+		warning("Unable to set socket pacing, using application pacing instead");
+		test->no_fq_socket_pacing = 1;
+	    }
+	}
+    }
+#endif /* HAVE_SO_MAX_PACING_RATE */
+
     /*
      * Create a new "listening" socket to replace the one we were using before.
      */
@@ -265,7 +322,7 @@ iperf_udp_connect(struct iperf_test *test)
 #endif
 
     /* Create and bind our local socket. */
-    if ((s = netdial(test->settings->domain, Pudp, test->bind_address, test->server_hostname, test->server_port)) < 0) {
+    if ((s = netdial(test->settings->domain, Pudp, test->bind_address, test->bind_port, test->server_hostname, test->server_port)) < 0) {
         i_errno = IESTREAMCONNECT;
         return -1;
     }
@@ -286,6 +343,23 @@ iperf_udp_connect(struct iperf_test *test)
         }
     }
 
+#if defined(HAVE_SO_MAX_PACING_RATE)
+    /* If socket pacing is available and not disabled, try it. */
+    if (! test->no_fq_socket_pacing) {
+	/* Convert bits per second to bytes per second */
+	unsigned int rate = test->settings->rate / 8;
+	if (rate > 0) {
+	    if (test->debug) {
+		printf("Setting fair-queue socket pacing to %u\n", rate);
+	    }
+	    if (setsockopt(s, SOL_SOCKET, SO_MAX_PACING_RATE, &rate, sizeof(rate)) < 0) {
+		warning("Unable to set socket pacing, using application pacing instead");
+		test->no_fq_socket_pacing = 1;
+	    }
+	}
+    }
+#endif /* HAVE_SO_MAX_PACING_RATE */
+
 #ifdef SO_RCVTIMEO
     /* 30 sec timeout for a case when there is a network problem. */
     tv.tv_sec = 30;
